# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-30 05:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('datatrans', '0003_add_modified_field'),
    ]

    operations = [
        migrations.AlterField(
            model_name='aliasregistration',
            name='card_alias',
            field=models.CharField(blank=True, db_index=True, max_length=20),
        ),
        migrations.AlterField(
            model_name='aliasregistration',
            name='masked_card_number',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]
